export = getLoader;
/**
 * Returns an object which provides methods to get the ids of the components which have to be loaded (`getIds`) and
 * a way to efficiently load them in synchronously and asynchronous contexts (`load`).
 *
 * The set of ids to be loaded is a superset of `load`. If some of these ids are in `loaded`, the corresponding
 * components will have to reloaded.
 *
 * The ids in `load` and `loaded` may be in any order and can contain duplicates.
 *
 * @param {Components} components
 * @param {string[]} load
 * @param {string[]} [loaded=[]] A list of already loaded components.
 *
 * If a component is in this list, then all of its requirements will also be assumed to be in the list.
 * @returns {Loader}
 *
 * @typedef Loader
 * @property {() => string[]} getIds A function to get all ids of the components to load.
 *
 * The returned ids will be duplicate-free, alias-free and in load order.
 * @property {LoadFunction} load A functional interface to load components.
 *
 * @typedef {<T> (loadComponent: (id: string) => T, chainer?: LoadChainer<T>) => T} LoadFunction
 * A functional interface to load components.
 *
 * The `loadComponent` function will be called for every component in the order in which they have to be loaded.
 *
 * The `chainer` is useful for asynchronous loading and its `series` and `parallel` functions can be thought of as
 * `Promise#then` and `Promise.all`.
 *
 * @example
 * load(id => { loadComponent(id); }); // returns undefined
 *
 * await load(
 *     id => loadComponentAsync(id), // returns a Promise for each id
 *     {
 *         series: async (before, after) => {
 *             await before;
 *             await after();
 *         },
 *         parallel: async (values) => {
 *             await Promise.all(values);
 *         }
 *     }
 * );
 */
declare function getLoader(components: any, load: string[], loaded?: string[] | undefined): Loader;
declare namespace getLoader {
    export { StringSet, EntryMap, DependencyResolver, LoadChainer, Loader, LoadFunction };
}
/**
 * Returns an object which provides methods to get the ids of the components which have to be loaded (`getIds`) and
 * a way to efficiently load them in synchronously and asynchronous contexts (`load`).
 *
 * The set of ids to be loaded is a superset of `load`. If some of these ids are in `loaded`, the corresponding
 * components will have to reloaded.
 *
 * The ids in `load` and `loaded` may be in any order and can contain duplicates.
 */
type Loader = {
    /**
     * A function to get all ids of the components to load.
     *
     * The returned ids will be duplicate-free, alias-free and in load order.
     */
    getIds: () => string[];
    /**
     * A functional interface to load components.
     */
    load: <T>(loadComponent: (id: string) => T, chainer?: LoadChainer<T> | undefined) => T;
};
/**
 * Returns a new set for the given string array.
 */
type StringSet = {
    [x: string]: true;
};
/**
 * Creates a map of every components id to its entry.
 */
type EntryMap = {
    readonly [id: string]: any;
};
/**
 * Creates a full dependencies map which includes all types of dependencies and their transitive dependencies.
 */
type DependencyResolver = (id: string) => {
    [x: string]: true;
};
type LoadChainer<T> = {
    series: (before: T, after: () => T) => T;
    parallel: (values: T[]) => T;
};
/**
 * A functional interface to load components.
 *
 * The `loadComponent` function will be called for every component in the order in which they have to be loaded.
 *
 * The `chainer` is useful for asynchronous loading and its `series` and `parallel` functions can be thought of as
 * `Promise#then` and `Promise.all`.
 */
type LoadFunction = <T>(loadComponent: (id: string) => T, chainer?: LoadChainer<T> | undefined) => T;
